"""Rebuilt everything

Revision ID: ce6d7c3b0d89
Revises:
Create Date: 2022-10-18 19:06:44.636596

"""
from alembic import op
import sqlalchemy as sa
import json
import backend
from backend.models.connection import DEFAULT_KEYWORD
import re
from backend.utils.misc import full_table_name, build_type
from backend.models import View, Card, Connection, UserConfig, UrlRule
from backend.models.url_rule import UrlRuleMatchingType, UrlRuleMode
from backend.models.card import CardType
from backend.models.connection import MultipleMatchesMode
from backend.models.user_config import UserConfigParam
from sqlalchemy import inspect


# revision identifiers, used by Alembic.
revision = "ce6d7c3b0d89"
down_revision = "630d977e6eb4"
branch_labels = None
depends_on = None


def map_legacy_nudge_type(legacy_type: str):
    type_mapping = {
        "text": "TEXT",
        "list": "LIST",
        "iframe": "IFRAME",
        "link": "LINK",
        "action": "LINK",
        "image": "IMAGE",
        "markdown": "MARKDOWN",
    }
    return type_mapping.get(legacy_type, "TEXT")


def map_legacy_json_paths(legacy_json_paths: str):
    data_columns = []
    json_paths_deserialized = json.loads(legacy_json_paths)
    for path in json_paths_deserialized:
        new_data_column_findall = re.findall(r"^\$\.results\[0\](?:(?:\[\"(.*)\"\])|(?:\.(.*)))$", path)
        if not new_data_column_findall:
            continue

        new_data_columns_valid = [c for c in new_data_column_findall[0] if c]
        if new_data_columns_valid:
            data_columns.append(new_data_columns_valid[0])
    return data_columns


def upgrade():
    conn = op.get_bind()
    current_dialect = conn.dialect.name
    inspector = inspect(conn)
    table_names = inspector.get_table_names()

    is_migration_from_old_data_model = full_table_name("state") in table_names

    if is_migration_from_old_data_model:
        if full_table_name("user_config") in table_names:
            op.drop_table(full_table_name("user_config"))

        if full_table_name("url_rule") in table_names:
            op.drop_table(full_table_name("url_rule"))

        if full_table_name("feedback") in table_names:
            op.drop_table(full_table_name("feedback"))

        if full_table_name("users") in table_names:
            op.drop_table(full_table_name("users"))

        if full_table_name("users_groups") in table_names:
            op.drop_table(full_table_name("users_groups"))

        if full_table_name("mapping_user_to_users_group") in table_names:
            op.drop_table(full_table_name("mapping_user_to_users_group"))

        if full_table_name("mapping_state_visibility_for_users_group") in table_names:
            op.drop_table(full_table_name("mapping_state_visibility_for_users_group"))

        if full_table_name("mapping_nudge_visibility_for_users_group") in table_names:
            op.drop_table(full_table_name("mapping_nudge_visibility_for_users_group"))

    if not View.__tablename__ in table_names:
        op.create_table(
            View.__tablename__,
            sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
            sa.Column("name", sa.Text(), nullable=False),
            sa.Column("is_active", sa.Boolean(), nullable=False),
            sa.PrimaryKeyConstraint("id"),
            keep_existing=True,
        )

    url_rule_matching_type = build_type(UrlRuleMatchingType, conn=conn, current_dialect=current_dialect)
    url_rule_mode = build_type(UrlRuleMode, conn=conn, current_dialect=current_dialect)

    if not UrlRule.__tablename__ in table_names:
        op.create_table(
            UrlRule.__tablename__,
            sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
            sa.Column("view_id", sa.Integer(), nullable=False),
            sa.Column(
                "matching_type",
                url_rule_matching_type,
                nullable=False,
            ),
            sa.Column("mode", url_rule_mode, nullable=False),
            sa.Column("priority", sa.Integer(), nullable=False),
            sa.Column("value", sa.String(length=64), nullable=True),
            sa.Column("css_selectors", backend.utils.misc.ArrayType(length=1000), nullable=True),
            sa.ForeignKeyConstraint(["view_id"], [f"{View.__tablename__}.id"], ondelete="CASCADE"),
            sa.PrimaryKeyConstraint("id"),
            keep_existing=True,
        )

    multiple_matches_mode = build_type(MultipleMatchesMode, conn=conn, current_dialect=current_dialect)
    if not Connection.__tablename__ in table_names:
        op.create_table(
            Connection.__tablename__,
            sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
            sa.Column("view_id", sa.Integer(), nullable=False),
            sa.Column("dataset_name", sa.Text(), nullable=True),
            sa.Column("keywords_column_name", sa.Text(), nullable=True),
            sa.Column(
                "multiple_matches_mode",
                multiple_matches_mode,
                nullable=True,
            ),
            sa.Column("secondary_column_name", sa.Text(), nullable=True),
            sa.Column("test_keyword_dict", sa.Text(), nullable=True),
            sa.ForeignKeyConstraint(["view_id"], [f"{View.__tablename__}.id"], ondelete="CASCADE"),
            sa.PrimaryKeyConstraint("id"),
            keep_existing=True,
        )

    card_type = build_type(CardType, conn=conn, current_dialect=current_dialect)
    if not Card.__tablename__ in table_names:
        op.create_table(
            Card.__tablename__,
            sa.Column("id", sa.Integer(), autoincrement=True, nullable=False),
            sa.Column("view_id", sa.Integer(), nullable=False),
            sa.Column("name", sa.Text(), nullable=False),
            sa.Column("type", card_type, nullable=False),
            sa.Column("data_columns", backend.utils.misc.ArrayType(length=1000), nullable=True),
            sa.Column("priority", sa.Integer(), nullable=True),
            sa.ForeignKeyConstraint(["view_id"], [f"{View.__tablename__}.id"], ondelete="CASCADE"),
            sa.PrimaryKeyConstraint("id"),
            keep_existing=True,
        )

    user_config_param = build_type(UserConfigParam, conn=conn, current_dialect=current_dialect)
    if not UserConfig.__tablename__ in table_names:
        op.create_table(
            UserConfig.__tablename__,
            sa.Column("dss_user_id", sa.String(length=64), nullable=False),
            sa.Column(
                "param",
                user_config_param,
                nullable=False,
            ),
            sa.Column("param_value", sa.Text(), nullable=False),
            sa.PrimaryKeyConstraint("dss_user_id", "param"),
            keep_existing=True,
        )

    if not is_migration_from_old_data_model:
        return

    view_table = sa.Table(View.__tablename__, sa.MetaData(), autoload=True, autoload_with=op.get_bind())
    connection_table = sa.Table(Connection.__tablename__, sa.MetaData(), autoload=True, autoload_with=op.get_bind())
    card_table = sa.Table(Card.__tablename__, sa.MetaData(), autoload=True, autoload_with=op.get_bind())

    old_state_table = sa.Table(full_table_name("state"), sa.MetaData(), autoload=True, autoload_with=op.get_bind())
    old_trigger_table = sa.Table(full_table_name("trigger"), sa.MetaData(), autoload=True, autoload_with=op.get_bind())
    old_nudge_table = sa.Table(full_table_name("nudge_def"), sa.MetaData(), autoload=True, autoload_with=op.get_bind())

    all_states = conn.execute(old_state_table.select()).mappings().fetchall()

    for state in all_states:
        state_trigger = (
            conn.execute(old_trigger_table.select().where(old_trigger_table.c.state_id == state["id"]))
            .mappings()
            .fetchone()
        )

        state_nudges = (
            conn.execute(old_nudge_table.select().where(old_nudge_table.c.state_id == state["id"]))
            .mappings()
            .fetchall()
        )

        new_connection = {
            "dataset_name": state_trigger["dataset_name"],
            "keywords_column_name": state_trigger["column_name"],
            "multiple_matches_mode": "KEEP_FIRST",
            "test_keyword_dict": json.dumps(DEFAULT_KEYWORD),
        }

        new_view = {"name": state["name"], "is_active": state["active"]}

        conn.execute(view_table.insert().values(new_view))
        inserted_view = (
            conn.execute(view_table.select().where(view_table.c.name == new_view["name"])).mappings().fetchall()
        )
        if not inserted_view:
            continue
        new_view_id = inserted_view[-1]["id"]

        new_connection["view_id"] = new_view_id

        conn.execute(connection_table.insert().values(new_connection))

        for nudge in state_nudges:
            new_data_columns = json.dumps(map_legacy_json_paths(nudge["json_paths"]))
            new_card = {
                "view_id": new_view_id,
                "name": nudge["name"],
                "type": map_legacy_nudge_type(nudge["type"]),
                "data_columns": new_data_columns,
                "priority": 50,
            }

            conn.execute(card_table.insert().values(new_card))

    op.drop_table(full_table_name("nudge_def"))
    op.drop_table(full_table_name("trigger"))
    op.drop_table(full_table_name("state"))
    op.drop_table(full_table_name("workspace"))
    op.drop_table(full_table_name("endpoint"))
    op.drop_table(full_table_name("integration"))


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table(UrlRule.__tablename__)
    op.drop_table(Connection.__tablename__)
    op.drop_table(Card.__tablename__)
    op.drop_table(View.__tablename__)
    op.drop_table(UserConfig.__tablename__)
    # ### end Alembic commands ###
