from flask import Blueprint, request, jsonify, Response

from backend.controllers import card_ctrl, auth_ctrl
from backend.schemas import CardSchema, DataColumnSchema, FedCardSchema
from backend.utils.misc import parse_req, NpEncoder
import json

card = Blueprint("card", __name__, url_prefix="/card")


@card.route("/<card_id>", methods=["GET"])
def get_by_id(card_id):
    card = card_ctrl.get_by_id(card_id)
    resp = {"card": CardSchema().dump(card)}
    return jsonify(resp)


@card.route("/list", methods=["GET"])
def list():
    cards = card_ctrl.get_all()
    return jsonify({"cards": CardSchema().dump(cards, many=True)})


@card.route("/<card_id>", methods=["DELETE"])
def delete(card_id):
    _ = card_ctrl.delete(card_id)
    return jsonify({"message": f"Card {card_id} deleted."})


@card.route("/<card_id>", methods=["PUT"])
def update(card_id):
    req_data = parse_req(
        request=request,
        optional_fields=["type", "name", "data_columns", "priority"],
        useless_fields=["id", "view_id", "view"],
    )
    updated_card = card_ctrl.update(card_id=card_id, **req_data)
    return jsonify({"card": CardSchema().dump(updated_card)})


@card.route("/<card_id>/data-columns", methods=["GET"])
def get_data_columns(card_id: int):
    data_columns = card_ctrl.get_data_columns(card_id)
    return jsonify({"data_columns": DataColumnSchema().dump(data_columns, many=True)})


@card.route("/<card_id>/data-columns", methods=["PUT"])
def update_data_columns(card_id: int):
    req_data = parse_req(request=request, required_fields=["data_columns"])
    updated_data_columns = card_ctrl.update_data_columns(card_id, **req_data)
    return jsonify({"data_columns": DataColumnSchema().dump(updated_data_columns, many=True)})


@card.route("/<card_id>/priority", methods=["PUT"])
def update_card_priority(card_id: int):
    req_data = parse_req(request=request, required_fields=["old_index", "new_index"])
    res = card_ctrl.update_card_priority(card_id, **req_data)
    return jsonify({"res": res})


@card.route("/create", methods=["POST"])
def create():
    req_data = parse_req(
        request=request,
        required_fields=["view_id", "name", "type", "priority"],
        optional_fields=["data_columns"],
    )
    created_card = card_ctrl.create(**req_data)
    return jsonify({"card": CardSchema().dump(created_card)})


@card.route("/<card_id>/test", methods=["GET"])
def test(card_id):
    current_user = auth_ctrl.get_current_dss_user(request)
    fed_card = card_ctrl.test_create_card(card_id=card_id, dss_user=current_user)
    fed_card_deserialized = FedCardSchema().dump(fed_card)
    return Response(json.dumps({"fed_card": fed_card_deserialized}, cls=NpEncoder))
