from flask import Blueprint, request, jsonify, Response
from backend.controllers import auth_ctrl, view_ctrl
from backend.schemas import (
    ViewSchema,
    UrlRuleSchema,
    FedCardSchema,
    ConnectionSchema,
    KeywordSchema,
    TriggerSchema,
    CardSchema,
)
from backend.utils.misc import parse_req, NpEncoder
from backend.utils.dataiku_api import dataiku_api
import backend.routes.api.connection as connection_route
import json
import logging

logger = logging.getLogger(__name__)

view = Blueprint("view", __name__, url_prefix="/view")


@view.route("/list", methods=["GET"])
def list():
    views = view_ctrl.get_all()
    return jsonify({"views": ViewSchema().dump(views, many=True)})


@view.route("/<view_id>", methods=["GET"])
def get_by_id(view_id: int):
    view = view_ctrl.get_by_id(view_id)
    return jsonify({"view": ViewSchema().dump(view)})


@view.route("/<view_id>/connection", methods=["GET"])
def get_view_connection(view_id: int):
    connection = view_ctrl.get_view_connection(view_id)
    return jsonify({"connection": ConnectionSchema().dump(connection)})


@view.route("/create", methods=["POST"])
def create():
    req_data = parse_req(request=request, required_fields=["name"], optional_fields=["description"])
    created_view = view_ctrl.create(**req_data)
    return_resp = jsonify({"view": ViewSchema().dump(created_view)})
    return_resp.status_code = 201
    return return_resp


@view.route("/<view_id>", methods=["PUT"])
def update(view_id: int):
    req_data = parse_req(
        request=request,
        optional_fields=["name", "description", "is_active"],
    )
    updated_view = view_ctrl.update(view_id=view_id, **req_data)
    return jsonify({"view": ViewSchema().dump(updated_view)})


@view.route("/<view_id>/connection", methods=["PUT"])
def update_connection(view_id: int):
    view = view_ctrl.get_by_id(view_id)
    response = connection_route.update(view.connection.id)
    return response


@view.route("/<view_id>", methods=["DELETE"])
def delete(view_id: int):
    has_been_deleted = view_ctrl.delete(view_id)
    return jsonify({"message": f"View {view_id} deleted."})


@view.route("/import", methods=["POST"])
def import_view():
    req_data = parse_req(
        request=request,
        required_fields=["name", "connection", "cards", "url_rules"],
        optional_fields=["description"],
    )
    imported_view = view_ctrl.import_(**req_data)
    return jsonify({"view": ViewSchema().dump(imported_view)})


@view.route("/<view_id>/export", methods=["GET"])
def export_view(view_id: int):
    view_to_export = view_ctrl.export(view_id)
    return jsonify({"view_for_export": view_to_export})


@view.route("/<view_id>/url-rules", methods=["GET"])
def get_url_rules(view_id: int):
    url_rules = view_ctrl.get_url_rules(view_id)
    return jsonify({"url_rules": UrlRuleSchema().dump(url_rules, many=True)})


@view.route("/<view_id>/cards", methods=["GET"])
def list_cards(view_id: int):
    cards = view_ctrl.get_view_cards(view_id=view_id)
    return jsonify({"cards": CardSchema().dump(cards, many=True)})


@view.route("/refresh-triggers-if-needed", methods=["POST"])  # DEPRECATED
@view.route("/refresh", methods=["POST"])
def refresh_if_needed():
    req_data = parse_req(request=request, required_fields=["refresh_hashes", "force"])
    triggers, active_triggers_id = view_ctrl.get_refreshed_and_active_triggers(**req_data)
    dumped = TriggerSchema().dump(triggers, many=True)
    try:
        plugin_config = dataiku_api.public_plugin_config
    except Exception as err:
        logger.error("Could not retrieve plugin config", err)
        plugin_config = {}
    resp = jsonify({"triggers": dumped, "active_triggers_id": active_triggers_id, "plugin_config": plugin_config})
    return resp


@view.route("/<view_id>/trigger", methods=["POST"])
def trigger(view_id: int):
    req_data = parse_req(request=request, required_fields=["keyword"])
    dss_user = auth_ctrl.get_current_dss_user(request)
    fed_cards = view_ctrl.feed_cards(view_id, dss_user=dss_user, **req_data)
    return jsonify({"fed_cards": FedCardSchema().dump(fed_cards, many=True)})


@view.route("/<view_id>/fetch-test-keyword-data", methods=["GET"])
def fetch_test_keyword_data(view_id: int):
    test_keyword_data = view_ctrl.fetch_test_keyword_data(view_id)
    return Response(json.dumps({"test_keyword_data": test_keyword_data}, cls=NpEncoder))


@view.route("/<view_id>/connection-test-keyword", methods=["GET"])
def fetch_test_keyword(view_id: int):
    test_keyword = view_ctrl.get_connection_test_keyword(view_id)
    return jsonify({"test_keyword": KeywordSchema().dump(test_keyword)})
