import flask_migrate
import os
from flask import Flask
import logging
from flask_migrate import Migrate
from backend.utils.dataiku_api import dataiku_api
from backend.utils.misc import get_webapp_id, extend_dku_placeholders

from flask_sqlalchemy import SQLAlchemy

from flask_cors import CORS

from werkzeug.exceptions import HTTPException
from marshmallow import ValidationError

from flask import jsonify, Flask

import traceback
import json

logger = logging.getLogger(__name__)

_WEBAPP_ID_PLACEHOLDER_KEY = "webAppId"
_DEFAULT_ADVISOR_FOLDER_NAME = "Advisor Settings"
_DEFAULT_ADVISOR_ZONE_NAME = "Advisor Settings"
_DEFAULT_DB_PREFIX = "$_{projectKey}_$_{" + _WEBAPP_ID_PLACEHOLDER_KEY + "}_"

def upgrade_db():
    current_directory = os.path.dirname(os.path.abspath(__file__))
    migration_path = os.path.join(current_directory, "../migrations")
    flask_migrate.upgrade(directory=migration_path)


def set_db_prefix_env():
    webapp_id = get_webapp_id()
    config = dataiku_api.webapp_config

    # Migrating from legacy config
    if config.get("use_table_prefix") == None:
        legacy_prefix = config.get("connections", {}).get("db_prefix")
        dataiku_api.set_webapp_config(webapp_id, {
            "use_table_prefix": not not legacy_prefix,
            "table_prefix": legacy_prefix if legacy_prefix else _DEFAULT_DB_PREFIX
        })

    config = dataiku_api.webapp_config

    if not config.get("use_table_prefix"):
        return

    tables_prefix = config.get("table_prefix", "")
    extra = {}
    extra[_WEBAPP_ID_PLACEHOLDER_KEY] = webapp_id

    db_prefix = extend_dku_placeholders(tables_prefix, **extra)
    
    logger.info(f"Prefixing tables with \"{db_prefix}\".")
    os.environ["DB_PREFIX"] = db_prefix


def get_or_create_advisor_settings_folder():
    dataiku_settings_folder = dataiku_api.get_or_create_managed_folder(
        _DEFAULT_ADVISOR_FOLDER_NAME, flow_zone_name=_DEFAULT_ADVISOR_ZONE_NAME
    )
    return dataiku_settings_folder


def get_conn_url():
    config = dataiku_api.webapp_config
    config_storage_location = config.get("config_storage_location")
    if config_storage_location is None:
        webapp_id = get_webapp_id()
        config_storage_location = "external"
        dataiku_api.set_webapp_config(
            webapp_id=webapp_id,
            new_config={
                "config_storage_location": config_storage_location,
            }
        )

    if config_storage_location == "advisor_settings_folder":
        dataiku_settings_folder = get_or_create_advisor_settings_folder()
        if dataiku_settings_folder is None:
            raise Exception(
                f"Could not retrieve information about folder {_DEFAULT_ADVISOR_FOLDER_NAME}. Are you sure you have a filesystem connection setup on the instance?"
            )
        folder_path = dataiku_settings_folder.get_path()
        db_conn_url = get_sqlite_url_from_path(folder_path)
    elif config_storage_location == "filesystem_folder":
        filesystem_folder_id = config.get("config_managed_folder")
        if filesystem_folder_id is None:
            raise Exception("You must provide a folder in which to store the config data of Advisor")
        dataiku_settings_folder = dataiku_api.get_managed_folder(filesystem_folder_id)
        if dataiku_settings_folder is None:
            raise Exception(f"The folder {filesystem_folder_id} does not exist.")
        try:
            folder_path = dataiku_settings_folder.get_path()
        except Exception as err:
            raise Exception(
                "The folder you selected might not be on the local filesystem."
                "Advisor does not support the usage of an external managed folder to store the config."
                f"\n Original error: {err}"
            )
        db_conn_url = get_sqlite_url_from_path(folder_path)
    else:
        db_conn_url = get_conn_url_from_preset()

    return db_conn_url


def get_sqlite_url_from_path(path: str) -> str:
    webapp_id = get_webapp_id()

    db_dir = os.path.abspath(os.path.expanduser(path))
    db_dir = extend_dku_placeholders(db_dir, **{_WEBAPP_ID_PLACEHOLDER_KEY: webapp_id})
    if not os.path.exists(db_dir):
        logger.info(f"Path {db_dir} does not exist. Creating it.")
        os.makedirs(db_dir)
    sqlite_conn_url = f"sqlite:///{os.path.join(db_dir, 'advisor.db')}"
    return sqlite_conn_url


def get_conn_url_from_preset():
    config = dataiku_api.webapp_config
    preset_config = config["connections"]
    storing_method = preset_config["storing_method"]

    if storing_method == "postgres":
        db_conn_name = preset_config["postgres_connection"]
        db_conn = dataiku_api.get_connection_info(db_conn_name)["params"]
        db_conn_url = (
            f"postgresql://{db_conn['user']}:{db_conn['password']}@{db_conn['host']}:{db_conn['port']}/{db_conn['db']}"
        )
    elif storing_method == "mysql":
        db_conn_name = preset_config["mysql_connection"]
        db_conn = dataiku_api.get_connection_info(db_conn_name)["params"]
        db_conn_url = f"mysql+pymysql://{db_conn['user']}:{db_conn['password']}@{db_conn['host']}:{db_conn['port']}/{db_conn['db']}"
    else:
        db_conn_url = get_sqlite_url_from_path(preset_config["fs_path"])
    return db_conn_url


def setup_app_for_db(app_to_setup: Flask, db: SQLAlchemy):
    db_conn_url = get_conn_url()

    app_to_setup.config["SQLALCHEMY_DATABASE_URI"] = db_conn_url
    app_to_setup.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = False

    _ = Migrate(app_to_setup, db)

    db.init_app(app_to_setup)


def run_migration_upgrade_func(client_setup=None):
    from backend.models import db

    migration_app = Flask("migration_app")
    if client_setup:
        dataiku_api.setup(**client_setup)
    setup_app_for_db(migration_app, db)
    _ = Migrate(migration_app, db)

    with migration_app.app_context():
        upgrade_db()

    return


def run_create_app(app: Flask, db: SQLAlchemy):

    from backend.routes import api_bp
    from backend.routes import main_bp

    setup_app_for_db(app, db)

    app.register_error_handler(HTTPException, handle_exception)
    app.register_error_handler(ValidationError, handle_validation_error)

    CORS(app)

    app.config["WEB_APP_ID"] = get_webapp_id()
    app.register_blueprint(api_bp)
    app.register_blueprint(main_bp)

    return app


def handle_exception(e):
    """Return JSON instead of HTML for HTTP errors."""
    response = e.get_response()
    data = {
        "code": e.code,
        "name": e.name,
        "type": e.name,
        "description": e.description,
        "details": getattr(e, "details", {}),
    }
    if os.getenv("FLASK_ENV") == "development":
        data["trace"] = traceback.format_exc()
    response.data = json.dumps(data)

    response.content_type = "application/json"
    return response


def handle_validation_error(e):
    error_fields = None
    generic_description = "Some fields are malformed: {error_fields}. See details field for more information."
    if type(e.messages) is dict:
        error_fields = ", ".join(e.messages.keys())
        description = generic_description.format(error_fields=error_fields)
        details = e.messages
    elif type(e.messages) is list and len(e.messages) == 1:
        description = e.messages[0]
        details = e.kwargs.get("details", {})
    else:
        description = "Validation Error. See details field for more information."
        details = e.messages

    data = {
        "code": 400,
        "name": "Bad Request",
        "type": "Validation Error",
        "description": description,
        "details": details,
    }
    if os.getenv("FLASK_ENV") == "development":
        data["trace"] = traceback.format_exc()
    response = jsonify(data)

    response.status = 400
    return response
